import prompts from 'prompts';
import path__default from 'path';
import chalk from 'chalk';
import hasbin from 'hasbin';
import arg from 'arg';
import { getLatestVersion, AppGenerator } from '@blitzjs/generator';
import { r as readVersions, l as loadEnvConfig } from './index2.mjs';
import { r as runPrisma } from './run-prisma.mjs';
import { i as isInternalBlitzMonorepoDevelopment, a as getGlobalPkgManager } from './helpers.mjs';
import { c as codegenTasks } from './codegen-tasks.mjs';
import 'cross-spawn';
import 'fs';
import 'dotenv';
import 'dotenv-expand';
import 'fs-extra';
import 'pkg-dir';
import 'resolve-cwd';
import 'resolve-from';
import 'p-event';
import './routes-manifest.mjs';
import 'os';
import 'watchpack';
import './find-node-modules.mjs';
import 'find-up';
import 'findup-sync';
import './logging.mjs';
import 'tslog';
import 'console-table-printer';
import 'ora';
import 'readline';
import 'semver';

var __async$1 = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const returnNpmEndpoint = (packageName) => {
  return `https://registry.npmjs.org/-/package/${packageName}/dist-tags`;
};
function getUpdateString(packageName, tag, isGlobal) {
  const globalPkgManager = getGlobalPkgManager();
  switch (globalPkgManager) {
    case "npm":
      return `npm install${isGlobal ? " -g" : ""} ${packageName}@${tag}`;
    case "yarn":
      return `yarn${isGlobal ? " global" : ""} add ${packageName}@${tag}`;
    case "pnpm":
      return `pnpm install${isGlobal ? " -g" : ""} ${packageName}@${tag}`;
  }
}
function checkLatestVersion() {
  return __async$1(this, null, function* () {
    if (!isInternalBlitzMonorepoDevelopment) {
      const fetch = yield import('node-fetch');
      const boxen = yield import('boxen');
      const versions = readVersions();
      const versionsArray = Object.entries(versions);
      let errors = [];
      try {
        const blitzResponse = yield fetch.default(returnNpmEndpoint("blitz"));
        const remoteBlitzVersions = yield blitzResponse.json();
        for (const version of versionsArray) {
          const versionType = version[0];
          const versionValue = version[1];
          if (versionType === "globalVersion" && typeof versionValue === "string" && versionValue !== "") {
            if (remoteBlitzVersions["latest"] !== versionValue) {
              errors.push({
                message: `blitz(global) (current) ${versionValue} -> (latest) ${remoteBlitzVersions["latest"]}`,
                instructions: `${getUpdateString("blitz", "latest", true)}`
              });
            }
          } else if (versionType === "localVersions" && typeof versionValue === "object" && versionValue.blitz && versionValue.blitz !== "") {
            if (remoteBlitzVersions["latest"] !== versionValue.blitz) {
              errors.push({
                message: `blitz(current) ${versionValue.blitz} -> (latest) ${remoteBlitzVersions["latest"]}`,
                instructions: `${getUpdateString("blitz", "latest", true)}`
              });
            }
          }
        }
        if (errors.length) {
          console.log(
            boxen.default(
              `You are running outdated blitz packages

${errors.map((e) => e.message).join("\n")} 

Run the following to update:
${errors.map((e) => e.instructions).join("\n")}`,
              { padding: 1 }
            )
          );
        }
      } catch (err) {
        if (err instanceof fetch.FetchError) ; else {
          console.log(err);
        }
      }
    }
  });
}

var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const forms = {
  formik: "Formik (Recommended)",
  finalform: "React Final Form",
  hookform: "React Hook Form"
};
const language = {
  typescript: "TypeScript"
};
const installCommandMap = {
  yarn: "yarn",
  pnpm: "pnpm install",
  npm: "npm install"
};
const templates = {
  app: {
    path: "app"
  },
  pages: {
    path: "pages"
  },
  minimal: {
    path: "minimalapp",
    skipForms: true,
    skipDatabase: true
  }
};
const IS_YARN_INSTALLED = hasbin.sync("yarn");
const IS_PNPM_INSTALLED = hasbin.sync("pnpm");
const PREFERABLE_PKG_MANAGER = IS_PNPM_INSTALLED ? "pnpm" : IS_YARN_INSTALLED ? "yarn" : "npm";
const args = arg(
  {
    "--npm": Boolean,
    "--yarn": Boolean,
    "--pnpm": Boolean,
    "--form": String,
    "--language": String,
    "--template": String,
    "--skip-install": Boolean,
    "--dry-run": Boolean,
    "--no-git": Boolean,
    "--skip-upgrade": Boolean
  },
  {
    permissive: true
  }
);
let projectName = "";
let projectPath = "";
let projectLanguage = language.typescript;
let projectFormLib = "formik";
let projectTemplate = templates.app;
let projectPkgManger = PREFERABLE_PKG_MANAGER;
let shouldInstallDeps = true;
const determineProjectName = () => __async(void 0, null, function* () {
  if (args._.slice(1).length < 1) {
    const res = yield prompts({
      type: "text",
      name: "name",
      message: "What would you like to name your project?",
      initial: "blitz-app"
    });
    projectName = res.name.trim().replaceAll(" ", "-");
    projectPath = path__default.resolve(projectName);
  } else {
    projectName = args._.slice(1)[0];
    if (projectName === ".") {
      projectName = path__default.basename(process.cwd());
    }
    projectPath = path__default.resolve(projectName);
  }
});
const determineFormLib = () => __async(void 0, null, function* () {
  if (!args["--form"] || args["--form"] && !Object.keys(forms).includes(args["--form"])) {
    const res = yield prompts({
      type: "select",
      name: "form",
      message: "Pick a form library (you can switch to something else later if you want)",
      initial: 0,
      choices: Object.entries(forms).map((c) => {
        return { value: c[0], title: c[1] };
      })
    });
    projectFormLib = res.form;
  } else {
    projectFormLib = args["--form"];
  }
});
const determineTemplate = () => __async(void 0, null, function* () {
  if (!args["--template"] || args["--template"] && !Object.keys(templates).includes(args["--template"].toLowerCase())) {
    const choices = [
      { value: "app", title: "App Router\xA0\u2013 Full Setup: includes DB and auth (Recommended)" },
      { value: "pages", title: "Pages Router \u2013 Full Setup: includes DB and auth" },
      { value: "minimal", title: "Pages Router\xA0\u2013 Minimal setup: no DB, no auth" }
    ];
    const res = yield prompts({
      type: "select",
      name: "template",
      message: "Pick your new app template",
      initial: 0,
      choices
    });
    projectTemplate = templates[res.template];
  } else {
    projectTemplate = templates[args["--template"]];
  }
});
const determinePkgManagerToInstallDeps = () => __async(void 0, null, function* () {
  if (args["--skip-install"]) {
    shouldInstallDeps = false;
    return;
  }
  const isPkgManagerSpecifiedAsFlag = args["--npm"] || args["--yarn"] || args["--pnpm"];
  if (isPkgManagerSpecifiedAsFlag) {
    if (args["--npm"]) {
      projectPkgManger = "npm";
    } else if (args["--pnpm"]) {
      if (IS_PNPM_INSTALLED) {
        projectPkgManger = "pnpm";
      } else {
        console.warn(`Pnpm is not installed. Fallback to ${projectPkgManger}`);
      }
    } else if (args["--yarn"]) {
      if (IS_YARN_INSTALLED) {
        projectPkgManger = "yarn";
      } else {
        console.warn(`Yarn is not installed. Fallback to ${projectPkgManger}`);
      }
    }
  } else {
    const hasPkgManagerChoice = IS_YARN_INSTALLED || IS_PNPM_INSTALLED;
    if (hasPkgManagerChoice) {
      const res = yield prompts({
        type: "select",
        name: "pkgManager",
        message: "Install dependencies?",
        initial: 0,
        choices: [
          { title: "npm", value: "npm" },
          { title: "yarn", value: "yarn", disabled: !IS_YARN_INSTALLED },
          { title: "pnpm", value: "pnpm", disabled: !IS_PNPM_INSTALLED },
          { title: "skip", value: "skip" }
        ]
      });
      if (res.pkgManager === "skip") {
        projectPkgManger = PREFERABLE_PKG_MANAGER;
      } else {
        projectPkgManger = res.pkgManager;
      }
      shouldInstallDeps = res.pkgManager !== "skip";
    } else {
      const res = yield prompts({
        type: "confirm",
        name: "installDeps",
        message: "Install dependencies?",
        initial: true
      });
      shouldInstallDeps = res.installDeps;
    }
  }
});
const newApp = () => __async(void 0, null, function* () {
  const shouldUpgrade = !args["--skip-upgrade"];
  if (shouldUpgrade) {
    yield checkLatestVersion();
  }
  yield determineProjectName();
  yield determineTemplate();
  yield determinePkgManagerToInstallDeps();
  if (!projectTemplate.skipForms) {
    yield determineFormLib();
  }
  try {
    const latestBlitzVersion = (yield getLatestVersion("blitz")).value;
    const requireManualInstall = args["--dry-run"] || !shouldInstallDeps;
    const postInstallSteps = projectName === "." ? [] : [`cd ${projectName}`];
    const generatorOpts = {
      template: projectTemplate,
      destinationRoot: projectPath,
      appName: projectName,
      useTs: projectLanguage === "TypeScript",
      yarn: projectPkgManger === "yarn",
      pnpm: projectPkgManger === "pnpm",
      dryRun: args["--dry-run"] ? args["--dry-run"] : false,
      skipGit: args["--no-git"] ? args["--no-git"] : false,
      skipInstall: !shouldInstallDeps,
      version: latestBlitzVersion,
      form: projectFormLib,
      onPostInstall: () => __async(void 0, null, function* () {
        if (projectTemplate.skipDatabase) {
          return;
        }
        try {
          loadEnvConfig(
            process.cwd(),
            void 0,
            { error: console.error, info: () => {
            } },
            { ignoreCache: true }
          );
          const result = yield runPrisma(["migrate", "dev", "--name", "Initial migration"], true);
          if (!result.success)
            throw new Error();
          if (projectPkgManger === "yarn") {
            yield codegenTasks();
          }
        } catch (error) {
          postInstallSteps.push(
            "blitz prisma migrate dev (when asked, you can name the migration anything)"
          );
        }
      })
    };
    const generator = new AppGenerator(generatorOpts);
    console.log("Hang tight while we set up your new Blitz app!");
    yield generator.run();
    if (requireManualInstall) {
      postInstallSteps.push(installCommandMap[projectPkgManger]);
      postInstallSteps.push(
        "blitz prisma migrate dev (when asked, you can name the migration anything)"
      );
    }
    postInstallSteps.push(`${projectPkgManger} blitz dev`);
    console.log("\n Your new Blitz app is ready! Next steps:");
    postInstallSteps.forEach((step, index) => {
      console.log(chalk.yellow(`   ${index + 1}. ${step}`));
    });
    console.log("");
    process.exit(0);
  } catch (error) {
    console.error(error);
    process.exit(1);
  }
});

export { newApp };
